/*
     File:       DICOM_Toolkit_Codec.h

     Contains:   Escape DICOM Toolkit Decompressor Component Interfaces.

     Copyright:  (c)2002-2004 by Escape Information Services, all rights reserved.

     Bugs?       For support or bug reports please send email to support@escape.gr
 
*/
#ifndef __DICOM_TOOLKIT_CODEC__
#define __DICOM_TOOLKIT_CODEC__

#ifndef __MACH__
	#ifndef __COMPONENTS__
	    #include <Components.h>
	#endif
	#ifndef __QUICKDRAW__
	    #include <QuickDraw.h>
	#endif
#endif

#ifndef __DICOM_TOOLKIT_TYPES__
	#include "DICOM_Toolkit_Types.h"
#endif

#ifdef __cplusplus
extern "C" {
#endif

#if PRAGMA_IMPORT
#pragma import on
#endif


/*!
    @header DICOM_Toolkit_Codec.h
    @discussion This header includes the declarations of the private functions of the DICOM Decompressor component
*/

#pragma export on

/*!
    @function DTkCodecVOIBlitterInit
    @abstract Initializes the VOI blitter operations on gray-scale images
    @discussion DTkCodecVOIBlitterInit should be called at the beginning of any VOI blitter operation, 
    			but after the image data have been decompressed. 
                Among other things it sets up the correspondence between the pixel parameters as defined
                in the file and those of the decompressor's internal representation.
                
                The function will return a kDTkBadVOIBlitterStateErr error if it is called before  
                the image data have been decompressed. 
    @param ci          [I] The decompressor component instance
    @param pixelFormat [I] Specifies the type of the source (gray-scale) DICOM image. Allowd values are: k16GrayCodecType and k8IndexedPixelFormat.
    @result Error code
*/
EXTERN_API(ComponentResult) DTkCodecVOIBlitterInit(
                                                     ComponentInstance ci, 
                                                     FourCharCode pixelFormat );


/*!
    @function DTkCodecSetHints
    @abstract Sets flags affecting decompression of DICOM files
    @discussion Please see documentation of DTkCodecHints enumeration as to which flags can be set and when. 
                Note: hints can also be set by adding the DTkDecompressorHintsImageDescExtension 
                Image Description Extension to an Image Description.
    @param ci         [I] The decompressor component instance
    @param codecHints [I] The hints to be passed to the DICOM decompressor
    @result Error code
*/
EXTERN_API(ComponentResult) DTkCodecSetHints( 
                                              ComponentInstance ci,
                                              UInt32 codecHints );


/*!
    @function DTkCodecGetHints
    @abstract Retrieves flags affecting decompression of DICOM files
    @discussion Please see documentation of DTkCodecHints enumeration as to which flags can be set and when. 
                Note: This call alose retrieves hints that have been set by adding the DTkDecompressorHintsImageDescExtension 
                Image Description Extension to an Image Description.
    @param ci         [I] The decompressor component instance
    @param codecHints [O] The hints to be retrieved from the DICOM decompressor
    @result Error code
*/
EXTERN_API(ComponentResult) DTkCodecGetHints( 
                                              ComponentInstance ci,
                                              UInt32* codecHints );


/*!
    @function DTkCodecGetPixelInfo
    @discussion Valid values for the selector are those specified in the DTkPixelInfoSelector enumeration.
                The correspondence between selector and type of the return value is given in the following table:
                <DIV  align = "CENTER">
                <TABLE border = "1" width="90%">
                    <thead> <TR><TH>Selector                              </TH><TH>Type                         </TH></TR></thead>
                            <TR><TD>kDTkPixelRepresentationsRecordSelector</TD><TD>DTkPixelRepresentationsRecord</TD></TR>
                            <TR><TD>kDTkPixelInversionStatusSelector      </TD><TD>DTkPixelInversionStatus      </TD></TR>
                            <TR><TD>kDTkFileVOIParamsSelector             </TD><TD>DTkFileVOIParams             </TD></TR>
                            <TR><TD>kDTkPhotometricInterpretationSelector </TD><TD>DTkPhotometricInterpretation </TD></TR>
                            <TR><TD>kDTkMinMaxPixelValuesSelector         </TD><TD>DTkMinMaxPixelValues         </TD></TR>
                            <TR><TD>kDTkPixelRangeShiftValueSelector      </TD><TD>DTkPixelRangeShiftValueType  </TD></TR>
                </TABLE>
                </DIV>
    @abstract Retrieves esoteric pixel information
    @param ci       [I] The decompressor component instance
    @param selector [I] Selector which specifies kind of information to be retrieved.
    @param value    [O] Pointer to a returned value appropriate to the selector
    @result Error code
*/
EXTERN_API(ComponentResult) DTkCodecGetPixelInfo( 
                                                  ComponentInstance ci, 
                                                  DTkPixelInfoSelector pixelInfoSelector, 
                                                  void * output );


/*!
    @function DTkCodecGetUserDefaultVOIParams
    @abstract Retrieves default VOI parameters (window center and window width) corresponding to the DICOM image
    @discussion Depending on the value of the selector parameter, DTkCodecGetUserDefaultVOIParams retrieves 
                a set of VOI parameters useful as default VOI parameter choices for displaying the image.
                Supported values of the selector parameter are:
                <UL> 
	                <LI> <b>0</b>: If VOI parameters exist in the file, the first set is retrieved. 
	                               If no VOI parameters are found in the file, optimal values are calculated from the 
	                               current image such that the after their application the image occupies the full available 
	                               8 bit output range. The formula used for calculating the optimal values is:<br>
	                               windowCenter = (maxPixelValue + minPixelValue) / 2<br>
	                               windowWidth = maxPixelValue - minPixelValue<br>
    	            <LI> <b>-1</b>: Instructs the call to return the optimal VOI parameters for the image, as for case 0. 
    	            <LI> <b>-2</b>: Instructs the call to return the VOI parameters that correspond to an identity VOI transformation
    	            <LI> <b>-3</b>: Instructs the call to return the VOI parameters that correspond the pixel bounds set by
                                    a previous invocation of the DTkCodecSetDisplayPixelBounds call. When no such values have been set the 
                                    kDTkDisplayPixelBoundsNotFoundErr
    	            <LI> <b>&gt;=1</b>: The selector is treated as an index to the VOI parameter sets of the DICOM file.
    	                                If the corresponding set exists in the file, it is retrieved; otherwise the 
    	                                kDTkVOIParamsNotFoundInFileErr error code is returned.
    	        </UL>
    @param ci              [I] The decompressor component instance
    @param usrWindowWidth  [O] Pointer to the first Window Width as it appears in the DICOM image file
    @param usrWindowCenter [O] Pointer to the first Window Level as it appears in the DICOM image file
    @param selector        [I] Specifies how default parameters will be chosen.
    @result Error code
*/
EXTERN_API(ComponentResult) DTkCodecGetUserDefaultVOIParams( 
                                                              ComponentInstance ci,
                                                              Float32 *usrWindowWidth, 
                                                              Float32 *usrWindowCenter,
                                                              SInt32   selector);

/*!
    @function DTkCodecGetUserVOIParamsFromDisplay
    @abstract Retrieves VOI parameters in the file pixel parameter reference system from decompressor
    @discussion DTkCodecGetUserVOIParamsFromDisplay converts the VOI parameters from their internal representation to
                values consistent with the pixel parameters used in the file.
    @param ci              [I] The decompressor component instance
    @param usrWindowWidth  [O] Pointer to currently used Window Width VOI parameter expressed in file units
    @param usrWindowCenter [O] Pointer to currently used Window Level VOI parameter expressed in file units
    @result Error code
*/
EXTERN_API(ComponentResult) DTkCodecGetUserVOIParamsFromDisplay(
                                                                  ComponentInstance ci,
                                                                  Float32 *usrWindowWidth, 
                                                                  Float32 *usrWindowCenter );


/*!
    @function DTkCodecGetUserVOILimits
    @abstract Retrieves the ranges of the VOI parameters expressed in the User Reference System
    @discussion This function is useful for setting the minimum and maximum values of e.g. slider controls to values that 
                are most meaningful for the particular DICOM file. These values take into account the actual range of 
                pixel values in the image instead of the nominal range as prescribed by the bit depth of the image. 
                Using the actual values results in an enhanced user experience, especially in situations where only a 
                small subset of the available range is actually use by the image.
    @param ci [I] The decompressor component instance
    @param minUsrWindowWidth  [O] Pointer to the minimum meaningful value of the Window Width
    @param maxUsrWindowWidth  [O] Pointer to the maximum meaningful value of the Window Width
    @param minUsrWindowCenter [O] Pointer to the minimum meaningful value of the Window Level
    @param maxUsrWindowCenter [O] Pointer to the maximum meaningful value of the Window Level
    @result Error code
*/
EXTERN_API(ComponentResult) DTkCodecGetUserVOILimits( 
                                                      ComponentInstance ci,
                                                      Float32 *minUsrWindowWidth, 
                                                      Float32 *maxUsrWindowWidth, 
                                                      Float32 *minUsrWindowCenter, 
                                                      Float32 *maxUsrWindowCenter );


/*!
    @function DTkCodecUseUserVOIParamsForDisplay
    @abstract Sets the user VOI parameters to be used for displaying the image
    @discussion DTkCodecUseUserVOIParamsForDisplay sets internal VOI parameters to values corresponding 
                to the input parameters, which are expressed in the File Reference System
    @param ci              [I] The decompressor component instance
    @param usrWindowWidth  [I] The input Window Width VOI parameter
    @param usrWindowCenter [I] The input Window Level VOI parameter
    @result Error code
*/
EXTERN_API(ComponentResult) DTkCodecUseUserVOIParamsForDisplay(
                                                                 ComponentInstance ci, 
                                                                 Float32 usrWindowWidth, 
                                                                 Float32 usrWindowCenter );


/*!
    @function DTkCodecDisplayToUserValue
    @abstract Converts a pixel value from the Display Reference System to the User Reference System
    @discussion DTkCodecDisplayToUserValue is a utility function that converts a pixel value from the 
                Display Reference System to the User Reference System. Do not use this functon to convert VOI parameters.
    @param ci        [I] The decompressor component instance
    @param dispValue [I] Pixel value expressed in the Display Reference System
    @param userValue [O] Pointer to returned pixel value expressed in the User Reference System
    @param flags	 [I] Currently only the kDTkDontCheckBounds flag is supported. When the flag is not set
                         the function returns an error when the dispValue is out of the allowed bounds corresponding 
                         to the pixel depth.    					
    @result Error code
*/
EXTERN_API(ComponentResult) DTkCodecDisplayToUserValue( 
                                                         ComponentInstance ci, 
                                                         SInt32 dispValue, 
                                                         Float32 *userValue,
                                                         UInt32 flags);

/*!
    @function DTkCodecUserToDisplayValue
    @abstract Converts a pixel value from the User Reference System to the Display Reference System
    @discussion DTkCodecDisplayToUserValue is a utility function that converts a pixel value from the 
                User Reference System to the Display Reference System. Do not use this functon to convert VOI parameters.
    @param ci        [I] The decompressor component instance
    @param userValue [I] Pixel value expressed in the User Reference System
    @param dispValue [O] Pointer to returned pixel value expressed in the Display Reference System
    @param flags	 [I] Currently only the kDTkDontCheckBounds flag is supported. When the flag is not set
                         the function returns an error when the userValue is out of the allowed bounds corresponding 
                         to the image.
    @result Error code
*/
EXTERN_API(ComponentResult) DTkCodecUserToDisplayValue( 
                                                         ComponentInstance ci, 
                                                         Float32 userValue,
                                                         SInt32* dispValue, 
                                                         UInt32 flags);


/*!
    @function DTkCodecGetDisplayVOIParams
    @abstract Retrieves the currently active VOI parameters expressed in the Display Reference System
    @param ci                  [I] The decompressor component instance
    @param displayWindowWidth  [O] Pointer to the currently active value of the Window Width
    @param displayWindowCenter [O] Pointer to the currently active value of the Window Level
    @result Error code
*/
EXTERN_API(ComponentResult) DTkCodecGetDisplayVOIParams(
                                                          ComponentInstance ci, 
                                                          UInt32 *displayWindowWidth, 
                                                          SInt32 *displayWindowCenter );


/*!
    @function DTkCodecGetDisplayVOILimits
    @abstract Retrieves the ranges of the VOI parameters expressed in the Display Reference System
    @param ci                     [I] The decompressor component instance
    @param displayMinWindowWidth  [O] Pointer to the minimum meaningful value of the Window Width
    @param displayMinWindowCenter [O] Pointer to the maximum meaningful value of the Window Width
    @param displayMaxWindowWidth  [O] Pointer to the minimum meaningful value of the Window Level
    @param displayMaxWindowCenter [O] Pointer to the maximum meaningful value of the Window Level
    @result Error code
*/
EXTERN_API(ComponentResult) DTkCodecGetDisplayVOILimits( 
                                                         ComponentInstance ci,
                                                         UInt32* displayMinWindowWidth, 
                                                         SInt32* displayMinWindowCenter, 
                                                         UInt32* displayMaxWindowWidth, 
                                                         SInt32* displayMaxWindowCenter );


/*!
    @function DTkCodecSetDisplayVOIParams
    @abstract Sets the VOI parameters to be applied in the next VOI transformation expressed in the Display Reference System
    @param ci                  [I] The decompressor component instance
    @param displayWindowWidth  [I] The value of the Window Width to be made current
    @param displayWindowCenter [I] The value of the Window Level to be made current
    @result Error code
*/
EXTERN_API(ComponentResult) DTkCodecSetDisplayVOIParams(
                                                          ComponentInstance ci, 
                                                          UInt32 displayWindowWidth, 
                                                          SInt32 displayWindowCenter );


/*!
    @function DTkCodecSetDisplayPixelBounds
    @abstract Sets the minimum and maximum pixel values to be used by the toolkit.
    @discussion The minimum and maximum pixel values are useful to set the range that will be 
                mapped to the output range during a VOI transformation.
    @param ci                   [I] The decompressor component instance
    @param displayMinPixelValue [I] The minimum pixel value
    @param displayMaxPixelValue [I] The maximum pixel value
    @result Error code
*/
EXTERN_API(ComponentResult) DTkCodecSetDisplayPixelBounds(
                                                          ComponentInstance ci, 
                                                          SInt32 displayMinPixelValue,
                                                          SInt32 displayMaxPixelValue);

/*!
	@function DTkCodecSetVOIBoundsClippingOption
	@abstract Sets the clipping method to be used during VOI transformations.
   	@discussion When a VOI bliting function is invoked the values of the destination pixels are calculated according 
                the following pseudo-code (for simplicity of presentation rounding effects are not included):

				<code>
                VOIWindowStart = VOIwindowCenter - VOIwindowWidth/2	
                VOIWindowEnd   = VOIWindowStart + VOIwindowWidth - 1

                if sourcePixel <= VOIWindowStart
                    destintationPixel = 0
                else if sourcePixel >= VOIWindowEnd
                    destintationPixel = 255
                else
                	destintationPixel = (sourcePixel - VOIWindowStart) * 255 / (VOIWindowStart - VOIWindowEnd)
				</code>

         		When VOIWindowStart in less that the minimum Pixel Value or VOIWindowEnd is greater than the maximum pixel value,
         		the actual pixel range will be mapped to a sub-range of the available output range, resulting in a compressed image.
         		
                To avoid this effect the VOI copy functions can optionally apply the following optimization to the VOI bounds:

				<code>
                if VOIWindowStart < minPixelValue
				{
					VOIWindowStart = minPixelValue
				}

				if VOIWindowEnd > maxPixelValue
				{
					*VOIWindowEnd = maxPixelValue
				}
				</code>
                The optimization results in mapping the minimum pixel value to 0 and the maximum pixel value to 255, 
                thus maximizing the resulting image's contrast.

				The kDTkDontOptimizeVOIBounds flag parameter instructs the function not to apply this optimization

				The kDTkClipVOIBoundsToActualPixelBounds flag parameter instructs the function to apply the 
				optimization using the actual pixel value bounds, as extracte from the image data.
				
				The kDTkClipVOIBoundsToNominalPixelBounds flag parameter instructs the function to apply the 
				optimization using the nominal pixel value bounds, as calculated from the Bits Stored (0x0028,0x0101))
				attribute of the DICOM file.
				
				The kDTkDontOptimizeVOIBounds lag parameter instructs the function to apply the 
				optimization using the pixel value bounds that have been specified by the user of the toolkit
				using the DtkCodecSetDisplayPixelBounds call.

				Note that the clipping option can also be set by adding the DTkVOIBoundsClippingOptionImageDescExtension
				Image Description Extension to an Image Description.
    @param ci             [I] The decompressor component instance
    @param clippingOption [I] The clipping option. Allowed values are defined in the DTkVOIBoundsClippingOption enumeration.
    @result Error code
*/
EXTERN_API(ComponentResult) DTkCodecSetVOIBoundsClippingOption(
                                                          ComponentInstance ci, 
                                                          DTkVOIBoundsClippingOption clippingOption);


/*!
	@function DTkCodecGetVOIBoundsClippingOption
	@abstract Retrieves the currently active clipping method used during VOI transformations.
    @param ci             [I] The decompressor component instance
    @param clippingOption [O] The retrieved clipping option. Allowed values are defined in the DTkVOIBoundsClippingOption enumeration.
    @result Error code
*/
EXTERN_API(ComponentResult) DTkCodecGetVOIBoundsClippingOption(
                                                          ComponentInstance ci, 
                                                          DTkVOIBoundsClippingOption *clippingOption);

/*!
    @function DTkCodecVOICopyPixels16To8bits
    @abstract Blits a source 16 bit image buffer to a destination 8 bit image buffer applying the current VOI parameters
    @discussion DTkCodecVOICopyPixels16To8bits is the function that actually applies the VOI transformation for gray-scale
                images with a bit depth > 8. The VOI parameters applied are those that have been previously set by a call to 
                DTkCodecUseUserVOIParamsForDisplay or DTkCodecSetDisplayVOIParams.                
    @param ci              [I] The decompressor component instance
    @param srcPixMapHandle [I] A handle to the source pixel map
    @param dstPixMapHandle [I] A handle to the destination pixel map
    @param srcRect         [I] The address of the source rectangle
    @param dstRect         [I] The address of the destination rectangle
    @result Error code
*/
EXTERN_API(ComponentResult) DTkCodecVOICopyPixels16To8bits(
                                                             ComponentInstance ci,
                                                             PixMapHandle srcPixMapHandle, 
                                                             PixMapHandle dstPixMapHandle, 
                                                             Rect *srcRect, 
                                                             Rect *dstRect );


/*!
    @function DTkCodecVOICopyPixels8To8bits
    @abstract Blits a source 8 bit image buffer to a destination 8 bit image buffer applying the current VOI parameters
    @discussion DTkCodecVOICopyPixels8To8bits is the function that actually applies the VOI transformation for gray-scale
                images with a bit depth <= 8. The VOI parameters applied are those that have been previously set by a call to 
                DTkCodecUseUserVOIParamsForDisplay or DTkCodecSetDisplayVOIParams.
    @param ci              [I] The decompressor component instance
    @param srcPixMapHandle [I] A handle to the source pixel map
    @param dstPixMapHandle [I] A handle to the destination pixel map
    @param srcRect         [I] The address of the source rectangle
    @param dstRect         [I] The address of the destination rectangle
    @result Error code
*/
EXTERN_API(ComponentResult) DTkCodecVOICopyPixels8To8bits(
                                                            ComponentInstance ci,
                                                            PixMapHandle srcPixMapHandle, 
                                                            PixMapHandle dstPixMapHandle, 
                                                            Rect *srcRect, 
                                                            Rect *dstRect );


/*!
    @function DTkCodecGetPixelValue
    @abstract Retrieves the value of a Pixel located at a certain location before and after a VOI transformation 
              expressed in the Display Reference System
    @param ci                 [I] The decompressor component instance
    @param pixMap             [I] The handle the pixel map to which the pixel belongs
    @param x                  [I] The horizontal coordinate of the pixel
    @param y                  [I] The vertical coordinate of the pixel
    @param pValueBefVOI       [O] Pointer to the value of the pixel before the VOI transformation 
    @param pValueAftVOIScaled [O] Pointer to the value of the pixel after the VOI transformation 
    @result Error code
*/
EXTERN_API(ComponentResult) DTkCodecGetPixelValue(
                                                   ComponentInstance ci, 
                                                   PixMapHandle pixMap, 
                                                   SInt32 x, 
                                                   SInt32 y, 
                                                   UInt16 *pValueBefVOI, 
                                                   UInt16 *pValueAftVOIScaled );


/*!
    @function DTkCodecSetMaskParams
    @abstract Set the parameters to be used in a Mask subtraction operation
    @param maskParamSelector  [I] Selector that specifies the type of the parameter passed
    @param param              [I] A pointer to parameter (single value or record)
    @result Error code
*/
EXTERN_API(ComponentResult) DTkCodecSetMaskParams(
                                                   ComponentInstance ci,
												   DTkMaskParamSelector maskParamSelector,
												   void* param);


/*!
    @function DTkCodecGetMaskParams
    @abstract Get the parameters used in a Mask subtraction operation
	@discussion				  Depending on the value of maskParamSelector the returned handle will 
	                          have the following content:
	                          <ul>
	                          	<li>kDTkMaskParamSelector_MaskPixMap: The value of the PixMapHandle previously set with DTkCodecSetMaskParams.</li>
	                          		Defaults to a null handle.
	                          	<li>kDTkMaskParamSelector_DisplacementRec: The returned handle contains a DTkMaskDisplacementRec struct. Defaults to a DTkMaskDisplacementRec
                                                                           with xDisp = 0.0, yDisp = 0.0.</li>
	                          	<li>kDTkMaskParamSelector_MaskMethod: The returned handle contains the value of the mask method typed as DTkMaskMethod. 
	                          	                                      Defaults to kDTkMaskMethod_Unknown.</li>
	                          	<li>kDTkMaskParamSelector_MaskFlags: The returned handle contains the value of the mask flags.
	                          	                                     Defaults to 0.<</li>
	                          	<li>kDTkMaskParamSelector_UserDefinedSrcMskRange: The returned handle contains a DTkMaskUserDefinedSrcDifRangeRec struct.
	                          	                                                  Defaults to a struct with all fields set to zero.</li>
	                          </ul>
	                          Note: If a value has not been set by a call to DTkCodecSetMaskParams using the respective selector, a default value is returned.
    @param maskParamSelector  [I] Selector that specifies the type of the parameter passed
    @param param              [I] A pointer to a parameter handle (single value or record). 
                                  It is the application's resposnibiliy to dispose of the handle
    @result Error code
*/
EXTERN_API(ComponentResult) DTkCodecGetMaskParams(
                                                   ComponentInstance ci,
												   DTkMaskParamSelector maskParamSelector,
												   Handle* paramHandle);

#pragma export off

#ifdef PRAGMA_IMPORT_OFF
#pragma import off
#elif PRAGMA_IMPORT
#pragma import reset
#endif

#ifdef __cplusplus
}
#endif


#endif // __DICOM_TOOLKIT_CODEC__
